#!/usr/bin/env python
# Keygrab
# Copyright (C) 2009 Alexandre Quessy
# http://alexandre.quessy.net
# All rights reserved.
# 
# This file is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# Keygrab is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Keygrab. If not, see <http://www.gnu.org/licenses/>.
# 
# On Debian systems the full text of the GNU General Public License version 3
# can be found in the `/usr/share/common-licenses/GPL-3' file.

"""
Communication with Pure Data. 

The fudi module is taken from Toonloop 1.1.8
"""
from twisted.internet import reactor
from grabber import fudi

class PureDataSender(object):
    """
    Sends FUDI messages

    If its "sender" attribute is not None, it means it is connected.
    """
    def __init__(self, host="localhost", port=17777, try_again_delay=10.0):
        """
        @param try_again_delay: Time in seconds before trying again.
        """
        self.send_port = port
        self.send_host = host
        self.sender = None
        self.verbose = True
        self.try_again = True
        self.try_again_delay = try_again_delay
        d = self._init_sender()

    def _init_sender(self):
        deferred = fudi.create_FUDI_client(self.send_host, self.send_port)
        deferred.addCallback(self._on_connected)
        deferred.addErrback(self._on_error)
        return deferred

    def _on_connected(self, sender):
        if self.verbose:
            print "Pure Data sender:", sender
        self.sender = sender
        #self.sender.send_message("/ping", 1, 2.0, "bang")
        #if self.verbose:
        #    print "  sent ping"

    def _on_error(self, failure):
        print 'Pure Data server unavailable:', self.send_host, self.send_port
        # print "FUDI sender:", failure.getErrorMessage()
        if self.try_again:
            reactor.callLater(self.try_again_delay, self._init_sender)

    def send(self, selector, *args):
        # TODO: if cannot send, delete send and raise Error
        if self.sender is None:
            print "Cannot send to %s %s" % (self.send_host, self.send_port)
        else:
            try:
                self.sender.send_message(selector, *args)
            except Exception, e:
                print e.message
                #self.sender = None # Nooooo !

def start(host="localhost", port="17777"): 
    """
    Factory for the FUDI sender.
    """
    fudi.VERBOSE = True
    print("Starting Pure Data sender.")
    sender = PureDataSender(host, port)
    return sender

